import pyodbc
from getpass import getpass
from wddasylumclaims.DbAsylumClaims.Db import DbAsylumClaims

class Admin(DbAsylumClaims):
    def __init__(self):
        super().__init__()

    def connect(self):
        # Get username and password from user
        username = input("Username: ")
        password = getpass("Password: ")
        database = "asylumclaims"
        # Connect to database with login details
        return(super().connect(database,username,password))

    def createUserLogin(self):
        # Create read-only user login
        username = "dbuser"
        password = "AsylumClaims123"
        return(self.createLogin(username,password))

    def createLogin(self,username,password):
        # Create new login
        if (super().checkConnected()): # check database connection
            # Create new login with sql command
            return(super().sqlCommand("CREATE USER [" + username + "]" + \
                                        "WITH PASSWORD = '" + password + "';" + \
                                        #"ALTER ROLE [db_datareader] ADD MEMBER [" + username + "];"))
                                        "EXEC sp_addrolemember 'db_datareader', '"+username+"';"))
        else:
            return False

    def listUsers(self):
        if (super().checkConnected()): # check database connection
            return(super().sqlCommand("SELECT * from asylumclaims.sys.sql_logins"))
        else:
            return False
        
        
    def resetTables(self):
        if (super().checkConnected()): # check database connection
            # Confirm reset with user
            answer = None
            while answer not in ("y", "n"):
                answer = input("WARNING! Are you sure you want to reset the database? ALL data will be WIPED! (y/n): ")
                if answer == "y":
                    print("ok")
                    # Delete tables
                    success = self.deleteTables()
                    if (not success):
                        print ("Failed to delete some tables (Maybe they did not exist to start with)")
                    # Re-create tables
                    success = self.createTables()
                    if (not success):
                        print ("Database table reset failed. Failed to re-create tables.")
                    else:
                        print ("Tables re-created. Tables reset complete.")
                    return success
                elif answer == "n":
                    print("Database table reset cancelled")
                else:
                    print("Invalid input. Please enter y/n")
        else:
            return False

    def deleteTables(self):
        # Delete all tables
        if (super().checkConnected()): # check database connection
            # delete raw text table
            success1 = self.deleteTable("tblClaimsRawText")
            if (not success1):
                print ("Failed to delete tblClaimsRawText")
            # delete links table
            success2 = self.deleteTable("tblClaimsLinks")
            if (not success2):
                print ("Failed to delete tblClaimsLinks")
            # delete outcomes table
            success3 = self.deleteTable("tblClaimsOutcomes")
            if (not success3):
                print ("Failed to delete tblClaimsOutcomes")
            # check success of deletion
            if (success1 and success2 and success3):
                print ("All tables deleted")
                return True
            else:
                return False
        else:
            return False

    def deleteTable(self,table_name):
        if (super().checkConnected()): # check database connection
            # Confirm deletion of table
            answer = None
            while answer not in ("y", "n"):
                answer = input("WARNING! Are you sure you want to delete the table"+table_name+"? ALL data will be WIPED! (y/n): ")
                if answer == "y":
                    # Run delete table sql command
                    print("Deleting table")
                    return(super().sqlCommand("DROP TABLE "+table_name+";"))
                elif answer == "n":
                    print("Database table reset cancelled")
                else:
                    print("Invalid input. Please enter y/n")
        else:
            return False
            
    def createTables(self):
        if (super().checkConnected()): # check database connection
            # Create raw text table
            sqlValid = super().sqlCommand("CREATE TABLE tblClaimsRawText (\
                                ID int NOT NULL AUTO_INCREMENT,\
                                case_id TEXT,\
                                full_text TEXT,\
                                promulgation_date DATE,\
                                PRIMARY KEY (ID)\
                            );"
                            )
            if (not sqlValid):
                return False
            # Create links table
            sqlValid = super().sqlCommand("CREATE TABLE tblClaimsLinks (\
                                ID int NOT NULL AUTO_INCREMENT,\
                                link TEXT,\
                                PRIMARY KEY (ID)\
                            );"
                            )
            if (not sqlValid):
                return False
            # Create outcomes table
            sqlValid = super().sqlCommand("CREATE TABLE tblClaimsOutcomes (\
                                ID int NOT NULL AUTO_INCREMENT,\
                                case_id TEXT,\
                                promulgation_date DATE,\
                                sogi_case BIT,\
                                unsuccessful BIT,\
                                successful BIT,\
                                ambiguous BIT,\
                                country TEXT,\
                                date_of_birth DATE,\
                                outcome_known BIT,\
                                multiple_outcomes BIT,\
                                no_page_available BIT,\
                                PRIMARY KEY (ID)\
                            );"
                            )
            return sqlValid
        else:
            return False


from bs4 import BeautifulSoup
import requests
import csv
import pandas as pd
import re
import os
import feather
import win32com.client


def scrape_url_list(limit=None, backupFolder=None, backupRate=5, url_pre="https://tribunalsdecisions.service.gov.uk/utiac?page=", url_post="&search%5Bclaimant%5D=&search%5Bcountry%5D=&search%5Bcountry_guideline%5D=&search%5Bjudge%5D=&search%5Bquery%5D=&search%5Breported%5D=all&utf8=%E2%9C%93"):
    print("Starting grabbing url list from search url...")

    urls = {'case_links': []}

    backup_count = 0
    backup_count_max = backupRate  # backup every n pages

    max_pages = 935

    prevArrLen = 0

    startIdx = 1
    if (not limit == None):
        if (limit[0] > 0):
            startIdx = limit[0]

        if (limit[1] > 0):
            max_pages = limit[1]+1

    for p in range(startIdx, max_pages):
        page = url_pre + str(p) + url_post
        isConnectionSuccess = True
        try:
            resp = requests.get(page, timeout=3)
            resp.raise_for_status()
        except requests.exceptions.RequestException as err:
            print("OOps: Something Else", err)
            isConnectionSuccess = False
        except requests.exceptions.HTTPError as errh:
            print("Http Error:", errh)
            isConnectionSuccess = False
        except requests.exceptions.ConnectionError as errc:
            print("Error Connecting:", errc)
            isConnectionSuccess = False
        except requests.exceptions.Timeout as errt:
            print("Timeout Error:", errt)
            isConnectionSuccess = False

        if (not isConnectionSuccess):
            urls.setdefault('case_links', []).append("")
        else:
            soup = BeautifulSoup(resp.content, features="html.parser")

            tds = soup.find_all("td")
            for td in tds:
                if (td.get("class") == ["reported"] or td.get("class") == ["unreported"]):
                    a = td.find_all("a")[0]
                    link = a.get("href")
                    urls.setdefault('case_links', []).append(link)

            print("{}/{}".format(p, max_pages-1))

        newLinksCount = len(urls['case_links']) - prevArrLen
        prevArrLen = len(urls['case_links'])
        print("new links added: {}".format(newLinksCount))

        backup_count = backup_count + newLinksCount

        if backup_count > backup_count_max:
            print("Saving backup...")
            # store temperary csv and feather files after each page to backup data
            feather_urls = pd.DataFrame(urls)

            if (not backupFolder == None):
                # Check backup folder exists
                if (os.path.exists(backupFolder)):
                    feather_urls_path = backupFolder + '\\py_tmp_case_links.feather'
                    csv_urls_path = backupFolder + '\\py_tmp_case_links.csv'
                else:
                    raise Exception(
                        "Backup folder does not exist: {}".format(backupFolder))

            try:
                # Save backup feather file
                feather.write_dataframe(feather_urls, feather_urls_path)
                # Save backup csv file
                feather_urls.to_csv(csv_urls_path)
                print("Backup saved")
                backup_count = 0
            except IOError as e:
                errno, strerror = e.args
                print("I/O error({0}): {1}".format(errno, strerror))
                print(
                    "This is likely due to the file being open. Please make sure it is closed.")

    feather_urls = pd.DataFrame(urls)
    print("Process complete")
    return feather_urls


def scrape(url, div_name):
    try:
        resp = requests.get(url, timeout=3)
        resp.raise_for_status()
    except requests.exceptions.RequestException as err:
        print("OOps: Something Else", err)
        return False
    except requests.exceptions.HTTPError as errh:
        print("Http Error:", errh)
        return False
    except requests.exceptions.ConnectionError as errc:
        print("Error Connecting:", errc)
        return False
    except requests.exceptions.Timeout as errt:
        print("Timeout Error:", errt)
        return False

    soup = BeautifulSoup(resp.content, features="html.parser")

    for div in soup.find_all("a"):
        if (div.get("class") == [div_name]):
            return str(div)
    return False


def scrape_decision(url):
    try:
        resp = requests.get(url, timeout=3)
        resp.raise_for_status()
    except requests.exceptions.RequestException as err:
        print("OOps: Something Else", err)
        return False
    except requests.exceptions.HTTPError as errh:
        print("Http Error:", errh)
        return False
    except requests.exceptions.ConnectionError as errc:
        print("Error Connecting:", errc)
        return False
    except requests.exceptions.Timeout as errt:
        print("Timeout Error:", errt)
        return False

    soup = BeautifulSoup(resp.content, features="html.parser")

    for div in soup.find_all("div"):
        if (div.get("class") == ["decision-inner"]):
            print("html found")
            data = div.get_text()
            data = ' '.join(data.split())
            return data
    for div in soup.find_all("div"):
        if (div.get("class") == ["download-links"]):
            regex = r"https://.*?\.doc"
            doc_url = re.findall(regex, str(div))
            if (len(doc_url) > 0):
                print("doc found")
                val = get_doc_data(doc_url[0])
                if (val == False):
                    return False
                return(val)
    return False


def get_doc_data(url):
    # Download word document to packages 'data' folder temperaraly
    print("Downloading document...")
    try:
        resp = requests.get(url, stream=True, timeout=3)
        resp.raise_for_status()
    except requests.exceptions.RequestException as err:
        print("OOps: Something Else", err)
        return False
    except requests.exceptions.HTTPError as errh:
        print("Http Error:", errh)
        return False
    except requests.exceptions.ConnectionError as errc:
        print("Error Connecting:", errc)
        return False
    except requests.exceptions.Timeout as errt:
        print("Timeout Error:", errt)
        return False

    # Check backup directory is valid exists
    script_dir = os.path.dirname(os.path.realpath(__file__))
    if (os.path.exists(script_dir + "\\data")):
        doc_filepath = script_dir + '\\data\\tmp_web.doc'
    elif (os.path.exists(script_dir + "\\..\\data")):
        doc_filepath = script_dir + '\\..\\data\\tmp_web.doc'
    else:
        raise Exception(
            "Script is in unexpected location, cannot locate packages 'data' folder")

    try:
        with open(doc_filepath, 'wb') as f:
            for chunk in resp.iter_content(1024 * 1024 * 2):
                f.write(chunk)

        # Load data from saved word document
        # CURRENT WINDOWS ONLY AS REQUIRED WIN32
        # NEEDS OFFICE INSTALLED
        # Open word application
        try:
            print("Reading document...")
            word = win32com.client.Dispatch("Word.Application")
            word.Visible = False
            wb = word.Documents.Open(doc_filepath)
            doc = word.ActiveDocument
            data = doc.Range().Text

            # close word
            doc.Close()
            word.Quit()

            print("done.")
        except Exception as e:
            print(e)
            return False
    except IOError as e:
        errno, strerror = e.args
        print("I/O error({0}): {1}".format(errno, strerror))
        print("This is likely due to the file being open. Please make sure it is closed.")
        return False

    if os.path.exists(doc_filepath):
        os.remove(doc_filepath)
    else:
        print("Doc file does not exist")

    # TODO load data from word document without needing office installed
    # TODO load word docmument directly without saving to file first

    data = ' '.join(data.split())
    # TODO fix issue with encoding of data from word document (current doesn't save the csv properly)
    return data


def scrape_promulgation_date(url):
    try:
        resp = requests.get(url, timeout=3)
        resp.raise_for_status()
    except requests.exceptions.RequestException as err:
        print("OOps: Something Else", err)
        return False
    except requests.exceptions.HTTPError as errh:
        print("Http Error:", errh)
        return False
    except requests.exceptions.ConnectionError as errc:
        print("Error Connecting:", errc)
        return False
    except requests.exceptions.Timeout as errt:
        print("Timeout Error:", errt)
        return False

    soup = BeautifulSoup(resp.content, features="html.parser")

    for i, div in enumerate(soup.find_all("li")):
        if i == 5:
            for res in div.find_all('time'):
                res2 = res.get('timedate')
                return str(res2)
    return False


def scrape_header(url):
    try:
        resp = requests.get(url, timeout=3)
        resp.raise_for_status()
    except requests.exceptions.RequestException as err:
        print("OOps: Something Else", err)
        return False
    except requests.exceptions.HTTPError as errh:
        print("Http Error:", errh)
        return False
    except requests.exceptions.ConnectionError as errc:
        print("Error Connecting:", errc)
        return False
    except requests.exceptions.Timeout as errt:
        print("Timeout Error:", errt)
        return False

    soup = BeautifulSoup(resp.content, features="html.parser")

    if (soup.h1):
        return(soup.h1.get_text())

    return False


def scrape_urls(feather_urls, limit=None, backupFolder=None, backupRate=50, div_name="decision-inner", url_prefix="https://tribunalsdecisions.service.gov.uk"):
    rows_count, cols_count = feather_urls.shape
    if (not limit == None):
        if (limit[1] > 0 and limit[1] < rows_count):
            rows_count = limit[1]

    dataset = {'case_id': [], 'full_text': [], 'promulgation_date': []}

    print("Scrapping data from urls...")

    backup_count = 0
    backup_count_max = backupRate  # backup every n rows

    for index, row in feather_urls.iterrows():
        if (not limit == None):
            if ((index < (limit[0]-1)) and (limit[0] > 0)):
                continue

        # get url from feather data
        url_suffix = row['case_links']
        if (url_suffix == ""):
            print("invalid url")
            continue
        else:
            full_url = url_prefix + url_suffix

            header_html = scrape_header(full_url)
            decision_html = scrape_decision(full_url)
            promulgation_date = scrape_promulgation_date(full_url)

            if (header_html == False or decision_html == False or promulgation_date == False):
                continue

            dataset.setdefault('case_id', []).append(header_html)
            dataset.setdefault('full_text', []).append(decision_html)
            dataset.setdefault('promulgation_date', []
                               ).append(promulgation_date)

        backup_count = backup_count + 1
        if backup_count > backup_count_max:
            print("Saving backup...")
            # store temperary csv and feather files after each row to backup data
            feather_dataset = pd.DataFrame(dataset)

            if (not backupFolder == None):
                # Check backup folder exists
                if (os.path.exists(backupFolder)):
                    feather_dataset_path = backupFolder + '\\py_tmp_case_text.feather'
                    csv_dataset_path = backupFolder + '\\py_tmp_case_text.csv'
                else:
                    raise Exception(
                        "Backup folder does not exist: {}".format(backupFolder))

            try:
                # Save backup feather file
                feather.write_dataframe(feather_dataset, feather_dataset_path)
                # Save backup csv file
                feather_dataset.to_csv(csv_dataset_path)
                print("Backup saved")
                backup_count = 0
            except IOError as e:
                errno, strerror = e.args
                print("I/O error({0}): {1}".format(errno, strerror))
                print(
                    "This is likely due to the file being open. Please make sure it is closed.")

        print("{}/{}".format(index+1, rows_count))

        if (not limit == None):
            if ((index >= limit[1]-1) and (limit[1] > 0)):
                break

    feather_dataset = pd.DataFrame(dataset)

    print("Process complete")

    return feather_dataset


if __name__ == "__main__":
    url = "https://tribunalsdecisions.service.gov.uk/utiac/pa-05705-2017-a23c9180-6c47-40f3-9294-67754ec19a04"
    div_name = 'decision-inner'
    div_data = scrape(url, div_name)
    print(div_data)



import pyodbc
from getpass import getpass
from wddasylumclaims.DbAsylumClaims.Db import DbAsylumClaims

class User(DbAsylumClaims):
    def __init__(self):
        super().__init__()

    def connect(self):
        username = "dbuser"
        password = "AsylumClaims123"
        database = "asylumclaims"
        return(super().connect(database,username,password))


from wddasylumclaims.DbAsylumClaims.Admin import Admin
from wddasylumclaims.DbAsylumClaims.User import User
import argparse

def get_args():
    
    parser = argparse.ArgumentParser(description='Asylum Claim azsure database test')
    parser.add_argument('-a','--admin',type=str, default=False, help="weather to test admin account (needs login information for access)")
    
    args = vars(parser.parse_args())
    return args

def main():
    args = get_args()

    requestAdmin = args["admin"]

    # Initalise connection sucess returns
    connectSucessUser = False
    connectSucessAdmin = False
    connectSucess = False
    
    # Connected to database as user
    userDb = User()
    connectSucessUser = userDb.connect()

    if (requestAdmin):
        # Connect to database as admin
        adminDb = Admin()
        connectSucessAdmin = adminDb.connect()
        if (connectSucessAdmin and connectSucessUser):
            connectSucess = True
        else:
            connectSucess = False
    
    # Check connection to database
    if(connectSucess):
        if (requestAdmin):
            # Run admin tests
            success = adminDb.printTables()
            if (not success):
                raise Exception("Failed to print database tables as admin")
            # Reset tables in database
            success = adminDb.resetTables()
            if (not success):
                raise Exception("Failed to reset tables in database as admin")
            # Create user login
            #TODO check if user is recreated if already exists or creation fails
            #success = adminDb.createUserLogin()
            #if (not success):
            #    raise Exception("Failed to create user login as admin")
            adminDb.commit_close()
        if (success):
            # Add row to database
            userDb.addRow("tblClaimsLinks","http://www.brainjar.com/java/host/test.html")
            # TODO Delete row from database
            # Commit changes and close connection to database
            userDb.commit_close()
        else:
            # Failed to reset tables. Close connection to database without commiting changes.
            print ("Closing connection to database without commiting changes.")
            adminDb.close()
    else:
        print("Failed to connected to database")

if __name__ == "__main__":
    main()


from wddasylumclaims import WebScrape
import pandas as pd
import csv
import os
import requests
import feather
import re

def update_csv(url_google_sheet,csv_filepath):
    with open(csv_filepath, mode='w',encoding="utf-8",newline="") as csv_file:
        try:
            resp = requests.get(url_google_sheet,timeout=3)
            resp.raise_for_status()
        except requests.exceptions.RequestException as err:
            print ("OOps: Something Else",err)
            return False
        except requests.exceptions.HTTPError as errh:
            print ("Http Error:",errh)
            return False
        except requests.exceptions.ConnectionError as errc:
            print ("Error Connecting:",errc)
            return False
        except requests.exceptions.Timeout as errt:
            print ("Timeout Error:",errt)
            return False
        csv_reader = csv.DictReader(resp.iter_lines(decode_unicode='utf-8'), delimiter=',')
        fieldnames = csv_reader.fieldnames
        try:
            writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
            writer.writeheader()

            known_headers = csv_reader.fieldnames
            keywords = {}
            for h in known_headers:
                keywords[h] = []

            for row in csv_reader:
                newRow = {}
                for h in known_headers:
                    if (row[h]):
                        newRow[h] = row[h]
                writer.writerow(newRow)
        except IOError as e:
            errno, strerror = e.args
            print("I/O error({0}): {1}".format(errno,strerror))
            print("This is likely due to the file being open. Please make sure it is closed.")
            return False
        
    return True

def find_csv(csv_filepath):
    # initialise keywords dictionary with known headings

    with open(csv_filepath, mode='r') as csv_file:
        # read csv
        csv_reader = csv.DictReader(csv_file, delimiter=',')

        known_headers = csv_reader.fieldnames
        keywords = {}
        for h in known_headers:
            keywords[h] = []

        # load data into dictionary using headers
        for row in csv_reader:
            for h in known_headers:
                if (row[h]):
                    keywords.setdefault(h, []).append(row[h])
    return keywords


def find_google_sheet(url):
    try:
        resp = requests.get(url)
        resp.raise_for_status()
    except requests.exceptions.RequestException as err:
        print ("OOps: Something Else",err)
        return False
    except requests.exceptions.HTTPError as errh:
        print ("Http Error:",errh)
        return False
    except requests.exceptions.ConnectionError as errc:
        print ("Error Connecting:",errc)
        return False
    except requests.exceptions.Timeout as errt:
        print ("Timeout Error:",errt)
        return False
        
    csv_reader = csv.DictReader(resp.iter_lines(decode_unicode='utf-8'), delimiter=',')

    known_headers = csv_reader.fieldnames
    keywords = {}
    for h in known_headers:
        keywords[h] = []

    for row in csv_reader:
        for h in known_headers:
            if (row[h]):
                keywords.setdefault(h, []).append(row[h])
    return keywords

def search_all_feather(feather_dataset, keywords, backupFolder=None, backupRate=50):
    print("Organising results...")

    #outcomes_raw = findKeywordsFeather(feather_dataset, keywords)
    outcomes = {
        'case_id': [], 'promulgation_date': [], 'sogi_case': [],
        'unsuccessful': [], 'successful': [], 'ambiguous': [],
        'country': [], 'date_of_birth': [], 'outcome_known': [],
        'multiple_outcomes': [], 'no_page_available': []
    }

    backup_count = 0
    backup_count_max = backupRate # backup every n rows

    rows_count,cols_count = feather_dataset.shape

    for index, row in feather_dataset.iterrows():
        raw_data = row['full_text']
        promulgation_date = row['promulgation_date']

        keywordCount = {}
        keywordLoc = {}
        for h in keywords.keys():
            keywordCount[h] = 0
            keywordLoc[h] = []

        case_id = row['case_id']
        for h in keywords.keys():
            for k in keywords[h]:
                if (raw_data):
                    regex = r"(?i)\b"+re.escape(k)+r"\b"
                    if h == "country":
                        regex = r"(?i)\b"+re.escape(k)+r"\b[ ]['A-Z']"
                    idx = re.search(regex,raw_data)
                    if (idx != None):
                        idx = idx.start()
                        if (idx > -1):
                            #print("found {} at {}".format(k,idx))
                            keywordLoc.setdefault(h, []).append([idx,idx+len(k)])
                            keywordCount[h] = keywordCount[h] + 1

        sogi_case = ""
        country = ""
        date_of_birth = ""
        unsuccessful = ""
        successful = ""
        ambiguous = ""
        outcome_known = ""
        multiple_outcomes = ""

        if (keywordCount['country'] > 0):
            #country found
            charIndexStart = int(keywordLoc['country'][0][1])
            charIndexEnd = len(raw_data)
            if (len(raw_data) > charIndexStart + 50):
                charIndexEnd = charIndexStart + 50
            slicetext = raw_data[charIndexStart:charIndexEnd]

            regex = r"\b[A-Za-z]+\b"
            matches = re.findall(regex,slicetext)
            if len(matches) > 0:
                country = matches[0]

        if (keywordCount['dob'] > 0):
            #dob found
            charIndexStart = int(keywordLoc['dob'][0][1]) + 1
            charIndexEnd = len(raw_data)
            if (len(raw_data) > charIndexStart + 50):
                charIndexEnd = charIndexStart + 50
            slicetext = raw_data[charIndexStart:charIndexEnd]

            regex = r".*?\d{4}"
            matches = re.findall(regex,slicetext)
            if len(matches) > 0:
                date_of_birth = matches[0]

        if (keywordCount['sexual_orientation_case'] > 0 or keywordCount['gender_identity_case'] > 0 or  keywordCount['unknown_sogi_case'] > 0):
            #sexual_orientation_case found
            sogi_case = True
        elif (keywordCount['sexual_orientation_case'] == 0 or keywordCount['gender_identity_case'] == 0 or keywordCount['unknown_sogi_case'] == 0):
            sogi_case = False

        outcome_multi_count = 0

        if (keywordCount['unsuccessful'] > 0):
            unsuccessful = True
            outcome_multi_count = outcome_multi_count + 1
        elif (keywordCount['unsuccessful'] == 0):
            unsuccessful = False
        if (keywordCount['successful'] > 0):
            successful = True
            outcome_multi_count = outcome_multi_count + 1
        elif (keywordCount['successful'] == 0):
            successful = False
        if (keywordCount['ambiguous_outcome'] > 0):
            ambiguous = True
            outcome_multi_count = outcome_multi_count + 1
        elif (keywordCount['ambiguous_outcome'] == 0):
            ambiguous = False

        if ((not ambiguous == "") or (not successful == "") or (not unsuccessful == "")):
            if (outcome_multi_count > 1):
                multiple_outcomes = True
                outcome_known = False
            else:
                multiple_outcomes = False
                if(outcome_multi_count == 1):
                    outcome_known = True
                else:
                    outcome_known = False
        else:
            outcome_known = False

        if (outcome_known == False):
            print(case_id)
            if unsuccessful:
                charIndexStart = int(keywordLoc['unsuccessful'][0][0])
                charIndexEnd = len(raw_data)
                if (len(raw_data) > charIndexStart + 50):
                    charIndexEnd = charIndexStart + 50
                slicetext = raw_data[charIndexStart:charIndexEnd]
                print (slicetext)
            if successful:
                charIndexStart = int(keywordLoc['successful'][0][0])
                charIndexEnd = len(raw_data)
                if (len(raw_data) > charIndexStart + 50):
                    charIndexEnd = charIndexStart + 50
                slicetext = raw_data[charIndexStart:charIndexEnd]
                print (slicetext)
            if ambiguous:
                charIndexStart = int(keywordLoc['ambiguous_outcome'][0][0])
                charIndexEnd = len(raw_data)
                if (len(raw_data) > charIndexStart + 50):
                    charIndexEnd = charIndexStart + 50
                slicetext = raw_data[charIndexStart:charIndexEnd]
                print (slicetext)
            
            if (ambiguous or successful or unsuccessful):
                #TODO check reason for aposing decisions found
                print ("aposing decisions found")
            else:
                #TODO check reason decision not found
                print("no descision found")
                '''
                known_ignore_case_ids = ['[2019] UKUT 217']
                known_ignore_case = False
                for k_id in known_ignore_case_ids:
                    if (k_id == case_id):
                        known_ignore_case = True
                        break
                if (not known_ignore_case):
                    print("new case id with unknown decision not found")
                else:
                    print("ignored case id with unknown decision as desision was marked as ")
                '''
                    
        no_page_available = False
        if raw_data == False:
            no_page_available = True

        outcomes.setdefault('case_id', []).append(case_id)
        outcomes.setdefault('promulgation_date', []).append(promulgation_date)
        outcomes.setdefault('sogi_case', []).append(sogi_case)
        outcomes.setdefault('unsuccessful', []).append(unsuccessful)
        outcomes.setdefault('successful', []).append(successful)
        outcomes.setdefault('ambiguous', []).append(ambiguous)
        outcomes.setdefault('country', []).append(country)
        outcomes.setdefault('date_of_birth', []).append(date_of_birth)
        outcomes.setdefault('outcome_known', []).append(outcome_known)
        outcomes.setdefault('multiple_outcomes', []).append(multiple_outcomes)
        outcomes.setdefault('no_page_available', []).append(no_page_available)

        backup_count = backup_count + 1
        if backup_count > backup_count_max:
            print ("Saving backup...")
            # store temperary csv and feather files after each row to backup data
            feather_outcomes = pd.DataFrame(outcomes)

            if (not backupFolder == None):
                # check backup data exists
                if (os.path.exists(backupFolder)):
                    feather_outcomes_path = backupFolder + '\\py_tmp_case_outcomes.feather'
                    csv_outcomes_path = backupFolder + '\\py_tmp_case_outcomes.csv'
                else:
                    raise Exception ("Backup folder does not exist: {}".format(backupFolder))
            try:
                # Save backup feather file
                feather.write_dataframe(feather_outcomes, feather_outcomes_path)
                # Save backup csv file
                feather_outcomes.to_csv(csv_outcomes_path,index_label=False)
                print ("Backup saved")
                backup_count = 0
            except IOError as e:
                errno, strerror = e.args
                print("I/O error({0}): {1}".format(errno,strerror))
                print("This is likely due to the file being open. Please make sure it is closed.")

        print("{}/{}".format(index+1, rows_count))

    feather_outcomes = pd.DataFrame(outcomes)
    return feather_outcomes



import pyodbc

class DbAsylumClaims:
    def __init__(self):
        # initialise database variables
        self.server = 'asylumclaims.database.windows.net'
        self.driver= '{SQL Server}'
        self.isInitialised = False

    def connect(self,database,username,password):
        # Connect to database with credentials
        try:
            # Connect to database
            self.cnxn = pyodbc.connect('DRIVER='+self.driver+';SERVER='+self.server+
                                        ';PORT=1433;DATABASE='+database+';UID='+
                                        username+';PWD='+ password)
            # Setup cursor for sql commands
            self.cursor = self.cnxn.cursor()
            self.isInitialised = True
            return True
        except pyodbc.Error as ex:
            # Failed to connect to database
            print("Database error caught")
            sqlstate = ex.args[1]
            print(sqlstate)
            return False

    def checkConnected(self):
        # Check connection to database
        if (not self.isInitialised):
            print("Not connected to database. Have you run connect() method?")
        return (self.isInitialised)

    def sqlCommand(self,sql):
        if (self.checkConnected()): # check connection to database
            try:
                # Run sql command
                self.cursor.execute(sql)
                return True
            except pyodbc.Error as ex:
                # Failed to run sql command
                print("Database error caught")
                sqlstate = ex.args[1]
                print(sqlstate)
                return False
        else: 
            # Connection to database not initialised
            return False

    def addRow(self,table,values):
        if (self.checkConnected()):
            sql = "INSERT INTO " + table
            # comma seperated values (e.g. "val1,val2,val3,val4")
            # should include value for all columns in table
            sql = sql + "VALUES (" + values + ");"
            success = self.sqlCommand(sql)
            return success
        else:
            # Connection to database is not initialised
            return False

    def printTables(self):
        if (self.checkConnected()): # check connection to database
            # Print all tables in database
            for row in self.cursor.tables():
                print (row.table_name)
            return True
        else:
            # Connection to database not initialised
            return False

    def close(self):
        if (self.checkConnected()): # check connection to database
            try:
                print ("Closing database connection")
                # Close database connection
                self.cnxn.close()
                self.isInitialised = False
                print ("Database connection closed")
                return True
            except pyodbc.Error as ex:
                # close failed
                print("Database error caught")
                sqlstate = ex.args[1]
                print(sqlstate)
                return False
        else:
            # Connection to database not initialised
            return False

    def commit(self):
        # Commit changes to database
        if (self.checkConnected()): # check database connection
            try:
                # commit changes to database
                self.cnxn.commit()
                return True
            except pyodbc.Error as ex:
                # commit failed
                print("Database error caught")
                sqlstate = ex.args[1]
                print(sqlstate)
                return False
        else:
            return False

    def commit_close(self):
        # Commit changes and close database connection
        # Commit changes to database
        print ("Commiting changes to database")
        success = self.commit()
        if (not success):
            return False
        # Close database connection
        success = self.close()
        return success

